@@ -253,7 +253,8 @@ clock_set_timeout (ClockData *cd)
  		struct timeval tv;
 		gettimeofday (&tv, NULL);
  		timeouttime = (1000000 - tv.tv_usec)/1000+1;
- 		if (!cd->showseconds)
+ 		if (cd->format != CLOCK_FORMAT_UNIX &&
+		    !cd->showseconds)
  			timeouttime += 1000 * (59 - cd->current_time % 60);
  	}
 	
@@ -2824,7 +2825,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 			  G_CALLBACK (properties_response_cb),
                           cd);
 
-        update_properties_for_format (cd, GTK_COMBO_BOX (combo), cd->format);
+	cd->custom_format_shown = FALSE;
+	update_properties_for_format (cd, GTK_COMBO_BOX (combo), cd->format);
 
 	/* valid values begin from 1 */
 	if (cd->can_handle_format_12)
