@@ -158,11 +158,12 @@ applet_scroll (PanelApplet    *applet,
                GdkEventScroll *event,
                PagerData      *pager)
 {
-	WnckScreen *screen;
-	int         index;
-	int         n_workspaces;
-	int         n_columns;
-	int         in_last_row;
+	WnckScreen         *screen;
+	GdkScrollDirection  absolute_direction;
+	int                 index;
+	int                 n_workspaces;
+	int                 n_columns;
+	int                 in_last_row;
 	
 	if (event->type != GDK_SCROLL)
 		return FALSE;
@@ -175,7 +176,22 @@ applet_scroll (PanelApplet    *applet,
 		n_columns++;
 	in_last_row    = n_workspaces % n_columns;
 	
-	switch (event->direction) {
+	absolute_direction = event->direction;
+	if (gtk_widget_get_direction (GTK_WIDGET (applet)) == GTK_TEXT_DIR_RTL) {
+		switch (event->direction) {
+		case GDK_SCROLL_DOWN:
+		case GDK_SCROLL_UP:
+			break;
+		case GDK_SCROLL_RIGHT:
+			absolute_direction = GDK_SCROLL_LEFT;
+			break;
+		case GDK_SCROLL_LEFT:
+			absolute_direction = GDK_SCROLL_RIGHT;
+			break;
+		}
+	}
+
+	switch (absolute_direction) {
 	case GDK_SCROLL_DOWN:
 		if (index + n_columns < n_workspaces)
 			index += n_columns;
@@ -606,7 +622,8 @@ update_workspaces_model (PagerData *pager)
 
 	nr_ws = wnck_screen_get_workspace_count (pager->screen);
         
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_workspaces_spin), nr_ws);
+	if (nr_ws != gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)))
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_workspaces_spin), nr_ws);
 
 	gtk_list_store_clear (pager->workspaces_store);
 	for (i = 0; i < nr_ws; i++) {
@@ -623,7 +640,16 @@ static void
 workspace_renamed (WnckWorkspace *space,
 		   PagerData     *pager)
 {
-	update_workspaces_model (pager);
+	int         i;
+	GtkTreeIter iter;
+
+	i = wnck_workspace_get_number (space);
+	if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pager->workspaces_store),
+					   &iter, NULL, i))
+		gtk_list_store_set (pager->workspaces_store,
+				    &iter,
+				    0, wnck_workspace_get_name (space),
+				    -1);
 }
 
 static void
@@ -653,8 +679,13 @@ workspace_destroyed (WnckScreen    *screen,
 
 static void
 num_workspaces_value_changed (GtkSpinButton *button,
-			      PagerData       *pager)
+			      PagerData     *pager)
 {
+	/* Slow down a bit after the first change, since it's moving really to
+	 * fast. See bug #336731 for background.
+	 * FIXME: remove this if bug 410520 gets fixed. */
+	button->timer_step = 0.2;
+
         wnck_screen_change_workspace_count (pager->screen,
                                             gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)));
 }
