@@ -884,6 +884,14 @@ restore_grabs(GtkWidget *w, gpointer data)
 	gtk_grab_add (GTK_WIDGET (menu));
 }
 
+static void
+menu_destroy_context_menu (GtkWidget *item,
+			   GtkWidget *menu)
+{
+	g_signal_handlers_disconnect_by_func (menu, restore_grabs, item);
+	gtk_widget_destroy (menu);
+}
+
 static GtkWidget *
 create_item_context_menu (GtkWidget   *item,
 			  PanelWidget *panel_widget)
@@ -918,13 +926,11 @@ create_item_context_menu (GtkWidget   *item,
 		return NULL;
 
 	menu = create_empty_menu ();
-	g_object_set_data_full (G_OBJECT (item),
-				"panel-item-context-menu",
-				g_object_ref (menu),
-				(GDestroyNotify) g_object_unref);
-	gtk_object_sink (GTK_OBJECT (menu));
-
+	g_object_set_data (G_OBJECT (item), "panel-item-context-menu", menu);
 	g_object_set_data (G_OBJECT (menu), "menu_panel", panel_widget);
+
+	g_signal_connect (item, "destroy",
+			  G_CALLBACK (menu_destroy_context_menu), menu);
 	g_signal_connect (menu, "deactivate",
 			  G_CALLBACK (restore_grabs), item);
 
@@ -1145,6 +1151,7 @@ image_menuitem_size_request (GtkWidget      *menuitem,
 {
 	GtkIconSize icon_size = (GtkIconSize) GPOINTER_TO_INT (data);
 	int         icon_height;
+	int         req_height;
 
 	if (!gtk_icon_size_lookup (icon_size, NULL, &icon_height))
 		return;
@@ -1152,8 +1159,13 @@ image_menuitem_size_request (GtkWidget      *menuitem,
 	/* If we don't have a pixmap for this menuitem
 	 * at least make sure its the same height as
 	 * the rest.
+	 * This is a bit ugly, since we should keep this in sync with what's in
+	 * gtk_menu_item_size_request()
 	 */
-	requisition->height = MAX (requisition->height, icon_height);
+	req_height = icon_height;
+	req_height += (GTK_CONTAINER (menuitem)->border_width +
+		       menuitem->style->ythickness) * 2;
+	requisition->height = MAX (requisition->height, req_height);
 }
 
 static char *
@@ -1231,7 +1243,7 @@ setup_menuitem (GtkWidget   *menuitem,
 		if (panel_menu_have_icons ())
 			gtk_image_menu_item_set_image (
 				GTK_IMAGE_MENU_ITEM (menuitem), image);
-	} else
+	} else if (icon_size != GTK_ICON_SIZE_INVALID)
 		g_signal_connect (menuitem, "size_request",
 				  G_CALLBACK (image_menuitem_size_request),
 				  GINT_TO_POINTER (icon_size));
