@@ -15,9 +15,9 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <libbonobo.h>
 #include <libgnomeui/gnome-icon-lookup.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -39,6 +39,7 @@
 #include "panel-applet-frame.h"
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
+#include "panel-separator.h"
 #include "panel-compatibility.h"
 #include "panel-multiscreen.h"
 #include "panel-toplevel.h"
@@ -96,6 +97,10 @@ orientation_change (AppletInfo  *info,
 				       panel_widget);
 		}
 		break;
+	case PANEL_OBJECT_SEPARATOR:
+		panel_separator_set_orientation (PANEL_SEPARATOR (info->widget),
+						 orientation);
+		break;
 	default:
 		break;
 	}
@@ -160,6 +165,9 @@ back_change (AppletInfo  *info,
 	case PANEL_OBJECT_MENU_BAR:
 		panel_menu_bar_change_background (PANEL_MENU_BAR (info->widget));
 		break;
+	case PANEL_OBJECT_SEPARATOR:
+		panel_separator_change_background (PANEL_SEPARATOR (info->widget));
+		break;
 	default:
 		break;
 	}
@@ -192,10 +200,8 @@ panel_back_change (GtkWidget *widget, gpointer data)
 static void
 panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
 {
-	PanelToplevel *toplevel;
 	AppletInfo    *info;
 
-	toplevel = PANEL_WIDGET (widget)->toplevel;
 	info = g_object_get_data (G_OBJECT (applet), "applet_info");
 
 	orientation_change(info,PANEL_WIDGET(widget));
@@ -260,15 +266,15 @@ static void
 panel_destroy (PanelToplevel *toplevel,
 	       PanelData     *pd)
 {
-	PanelWidget *panel_widget;
-
-	panel_widget = panel_toplevel_get_panel_widget (toplevel);
-
 	panel_lockdown_notify_remove (G_CALLBACK (panel_recreate_context_menu),
 				      pd);
 
-	if (pd->menu)
+	if (pd->menu) {
+		g_signal_handlers_disconnect_by_func (pd->menu,
+						      context_menu_deactivate,
+						      pd);
 		g_object_unref (pd->menu);
+	}
 	pd->menu = NULL;
 
 	pd->panel = NULL;
@@ -299,21 +305,18 @@ static GtkWidget *
 panel_menu_get (PanelWidget *panel, PanelData *pd)
 {
 	if (!pd->menu) {
-		pd->menu = g_object_ref (panel_context_menu_create (panel));
-		if (pd->menu) {
-			gtk_object_sink (GTK_OBJECT (pd->menu));
-			g_signal_connect (pd->menu, "deactivate",
-					  G_CALLBACK (context_menu_deactivate),
-					  pd);
-			g_signal_connect (pd->menu, "show",
-					  G_CALLBACK (context_menu_show), pd);
-		}
+		pd->menu = g_object_ref_sink (panel_context_menu_create (panel));
+		g_signal_connect (pd->menu, "deactivate",
+				  G_CALLBACK (context_menu_deactivate),
+				  pd);
+		g_signal_connect (pd->menu, "show",
+				  G_CALLBACK (context_menu_show), pd);
 	}
 
 	return pd->menu;
 }
 
-GtkWidget *
+static GtkWidget *
 make_popup_panel_menu (PanelWidget *panel_widget)
 {
 	PanelData *pd;
@@ -348,13 +351,9 @@ panel_popup_menu (PanelToplevel *toplevel,
 	panel_data   = g_object_get_data (G_OBJECT (toplevel), "PanelData");
 
 	current_event = gtk_get_current_event ();
-	if (current_event->type == GDK_BUTTON_PRESS) {
-		int x = -1, y = 1;
-	
-		gtk_widget_get_pointer (GTK_WIDGET (panel_widget), &x, &y);
-
-		panel_data->insertion_pos = panel_widget->orient == GTK_ORIENTATION_HORIZONTAL ? x : y;
-	} else
+	if (current_event->type == GDK_BUTTON_PRESS)
+		panel_data->insertion_pos = panel_widget_get_cursorloc (panel_widget);
+	else
 		panel_data->insertion_pos = -1;
 	
 	menu = make_popup_panel_menu (panel_widget);
@@ -538,6 +537,41 @@ drop_uri (PanelWidget *panel,
 	return TRUE;
 }
 
+static gboolean
+drop_nautilus_desktop_uri (PanelWidget *panel,
+			   int          pos,
+			   const char  *uri)
+{
+	gboolean    success;
+	const char *id;
+	const char *basename;
+
+	if (g_ascii_strncasecmp (uri, "x-nautilus-desktop:///",
+				 strlen ("x-nautilus-desktop:///")) != 0)
+			return FALSE;
+
+	success = TRUE;
+	id = panel_profile_get_toplevel_id (panel->toplevel);
+	basename = uri + strlen ("x-nautilus-desktop:///");
+
+	if (strncmp (basename, "trash", strlen ("trash")) == 0)
+		panel_applet_frame_create (panel->toplevel, pos,
+					   "OAFIID:GNOME_Panel_TrashApplet");
+	else if (strncmp (basename, "home", strlen ("home")) == 0)
+		panel_launcher_create_with_id (id, pos,
+					       "nautilus-home.desktop");
+	else if (strncmp (basename, "computer", strlen ("computer")) == 0)
+		panel_launcher_create_with_id (id, pos,
+					       "nautilus-computer.desktop");
+	else if (strncmp (basename, "network", strlen ("network")) == 0)
+		panel_launcher_create_with_id (id, pos,
+					       "nautilus-scheme.desktop");
+	else
+		success = FALSE;
+
+	return success;
+}
+
 static gboolean
 drop_urilist (PanelWidget *panel,
 	      int          pos,
@@ -556,18 +590,17 @@ drop_urilist (PanelWidget *panel,
 		GnomeVFSResult    res;
 		const char       *uri;
 		char             *basename;
-		char             *dirname;
 		char             *filename;
 
 		uri = uris[i];
 
-		if (strncmp (uri, "http:", strlen ("http:")) == 0 ||
-		    strncmp (uri, "https:", strlen ("https:")) == 0 ||
-		    strncmp (uri, "ftp:", strlen ("ftp:")) == 0 ||
-		    strncmp (uri, "gopher:", strlen ("gopher:")) == 0 ||
-		    strncmp (uri, "ghelp:", strlen ("ghelp:")) == 0 ||
-		    strncmp (uri, "man:", strlen ("man:")) == 0 ||
-		    strncmp (uri, "info:", strlen ("info:")) == 0) {
+		if (g_ascii_strncasecmp (uri, "http:", strlen ("http:")) == 0 ||
+		    g_ascii_strncasecmp (uri, "https:", strlen ("https:")) == 0 ||
+		    g_ascii_strncasecmp (uri, "ftp:", strlen ("ftp:")) == 0 ||
+		    g_ascii_strncasecmp (uri, "gopher:", strlen ("gopher:")) == 0 ||
+		    g_ascii_strncasecmp (uri, "ghelp:", strlen ("ghelp:")) == 0 ||
+		    g_ascii_strncasecmp (uri, "man:", strlen ("man:")) == 0 ||
+		    g_ascii_strncasecmp (uri, "info:", strlen ("info:")) == 0) {
 			/* FIXME: probably do this only on link,
 			 * in fact, on link always set up a link,
 			 * on copy do all the other stuff.  Or something. */
@@ -576,13 +609,18 @@ drop_urilist (PanelWidget *panel,
 			continue;
 		}
 
+		if (g_ascii_strncasecmp (uri, "x-nautilus-desktop:",
+					 strlen ("x-nautilus-desktop:")) == 0) {
+			success = drop_nautilus_desktop_uri (panel, pos, uri);
+			continue;
+		}
+
 		if (!(vfs_uri = gnome_vfs_uri_new (uri))) {
 			success = FALSE;
 			continue;
 		}
 
 		basename = gnome_vfs_uri_extract_short_path_name (vfs_uri);
-		dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
 
 		info = gnome_vfs_file_info_new ();
 		res = gnome_vfs_get_file_info_uri (vfs_uri, info,
@@ -639,7 +677,6 @@ drop_urilist (PanelWidget *panel,
 		gnome_vfs_file_info_unref (info);
 
 		g_free (basename);
-		g_free (dirname);
 
 		gnome_vfs_uri_unref (vfs_uri);
 	}
@@ -666,7 +703,8 @@ drop_internal_icon (PanelWidget *panel,
 	if (action == GDK_ACTION_MOVE)
 		old_launcher = find_launcher (icon_name);
 	
-	panel_launcher_create_copy (panel->toplevel, pos, icon_name);
+	if (!panel_launcher_create_copy (panel->toplevel, pos, icon_name))
+		return FALSE;
 
 	if (old_launcher && old_launcher->button) {
 		if (old_launcher->prop_dialog) {
@@ -746,7 +784,7 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 			char *menu_filename;
 
 			menu_filename = g_strndup (menu, menu_path - menu);
-			*menu_path++;
+			menu_path++;
 			success = drop_menu (panel, pos,
 					     menu_filename, menu_path);
 			g_free (menu_filename);
@@ -780,6 +818,14 @@ drop_internal_applet (PanelWidget *panel, int pos, const char *applet_type,
 			success = FALSE;
 		}
 
+	} else if (!strcmp(applet_type,"SEPARATOR:NEW")) {
+		if (panel_profile_id_lists_are_writable ()) {
+			panel_separator_create (panel->toplevel, pos);
+			success = TRUE;
+		} else {
+			success = FALSE;
+		}
+
 	} else if (!strcmp(applet_type,"LAUNCHER:ASK")) {
 		if (panel_profile_id_lists_are_writable ()) {
 			ask_about_launcher (NULL, panel, pos, TRUE);
@@ -1218,13 +1264,10 @@ panel_delete_without_query (PanelToplevel *toplevel)
 	if (panel_toplevel_get_is_attached (toplevel) &&
 	    panel_widget->master_widget) {
 		AppletInfo *info;
-		Drawer *drawer;
 
 		info = g_object_get_data (G_OBJECT (panel_widget->master_widget),
 					  "applet_info");
 
-		drawer = (Drawer *) info->data;
-
 		panel_profile_delete_object (info);
 	} else
 		panel_profile_delete_toplevel (toplevel);
@@ -1347,7 +1390,5 @@ panel_delete (PanelToplevel *toplevel)
 		return;
 	}
 
-	panel_toplevel_push_autohide_disabler (toplevel);
-
 	panel_query_deletion (toplevel);
 }
